@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2014-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -25,19 +25,27 @@ const St = imports.gi.St;
 
 const GPasteDeleteButton = new Lang.Class({
     Name: 'GPasteDeleteButton',
-    Extends: St.Button,
 
     _init: function(client, index) {
-        this.parent();
+        this.actor = new St.Button();
 
-        this.child = new St.Icon({
+        this.actor.child = new St.Icon({
             icon_name: 'edit-delete-symbolic',
-            style_class: 'system-status-icon'
+            style_class: 'popup-menu-icon'
         });
 
-        this.connect('clicked', function() {
-            client.delete(index, null);
-            return Clutter.EVENT_STOP;
-        });
+        this._client = client;
+        this.setIndex(index);
+
+        this.actor.connect('clicked', Lang.bind(this, this._onClick));
+    },
+
+    setIndex: function(index) {
+        this._index = index;
+    },
+
+    _onClick: function() {
+        this._client.delete(this._index, null);
+        return Clutter.EVENT_STOP;
     }
 });
