@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2014-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -24,20 +24,23 @@ const St = imports.gi.St;
 
 const GPasteEmptyHistoryItem = new Lang.Class({
     Name: 'GPasteEmptyHistoryItem',
-    Extends: St.Button,
 
     _init: function(client) {
-        this.parent({
+        this.actor = new St.Button({
             reactive: true,
             can_focus: true,
             track_hover: true,
             style_class: 'system-menu-action'
         });
 
-        this.child = new St.Icon({ icon_name: 'edit-clear-all-symbolic' });
+        this.actor.child = new St.Icon({ icon_name: 'edit-clear-all-symbolic' });
 
-        this.connect('clicked', function() {
-            client.empty(null);
+        this.actor.connect('clicked', function() {
+            client.get_history_name(Lang.bind(this, function(client, result) {
+                let name = client.get_history_name_finish(result);
+
+                client.empty_history(name, null);
+            }));
         });
     }
 });
