@@ -29,6 +29,13 @@ struct _GPasteClipboard
     GObject parent_instance;
 };
 
+typedef enum
+{
+    CLIP_STATE_IDLE,
+    CLIP_STATE_WAITING,
+    CLIP_STATE_BUSY
+} ClipState;
+
 typedef struct
 {
     GdkAtom         target;
@@ -38,6 +45,11 @@ typedef struct
     gchar          *image_checksum;
 
     gulong          owner_change_signal;
+
+    /* GtkClipboard has a bug where it overrides an internal GMainLoop
+     * if we call gtk_clipboard_store twice in less than 10s, resulting
+     * in a g_main_loop_quit (NULL) call making GPaste crash */
+    ClipState       state;
 } GPasteClipboardPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GPasteClipboard, g_paste_clipboard, G_TYPE_OBJECT)
@@ -61,7 +73,12 @@ g_paste_clipboard_bootstrap_finish (GPasteClipboard *self,
     {
         const GSList *h = g_paste_history_get_history (history);
         if (h)
-            g_paste_clipboard_select_item (self, h->data);
+        {
+            GPasteItem *item = h->data;
+
+            if (!g_paste_clipboard_select_item (self, item))
+                g_paste_history_remove (history, 0);
+        }
     }
 }
 
@@ -173,6 +190,17 @@ g_paste_clipboard_get_text (const GPasteClipboard *self)
     return priv->text;
 }
 
+static const gchar *
+_g_paste_clipboard_private_target_name (const GPasteClipboardPrivate *priv)
+{
+    if (priv->target == GDK_SELECTION_CLIPBOARD)
+        return "CLIPBOARD";
+    else if (priv->target == GDK_SELECTION_PRIMARY)
+        return "PRIMARY";
+    else
+        return "UNKNOWN";
+}
+
 static void
 g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
                                     const gchar            *text)
@@ -180,6 +208,8 @@ g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
     g_free (priv->text);
     g_free (priv->image_checksum);
 
+    g_debug("%s: set text", _g_paste_clipboard_private_target_name (priv));
+
     priv->text = g_strdup (text);
     priv->image_checksum = NULL;
 }
@@ -267,6 +297,47 @@ g_paste_clipboard_set_text (GPasteClipboard            *self,
                                 data);
 }
 
+static void g_paste_clipboard_private_store (GPasteClipboardPrivate *priv);
+
+static gboolean
+g_paste_clipboard_private_store_timeout (gpointer user_data)
+{
+    GPasteClipboardPrivate *priv = user_data;
+    gboolean waiting = (priv->state == CLIP_STATE_WAITING);
+
+    /* Reset us as IDLE */
+    priv->state = CLIP_STATE_IDLE;
+
+    /* If we were waiting, perform the store now */
+    if (waiting)
+        g_paste_clipboard_private_store (priv);
+
+    return G_SOURCE_REMOVE;
+}
+
+static void
+g_paste_clipboard_private_store (GPasteClipboardPrivate *priv)
+{
+    g_debug("%s: store", _g_paste_clipboard_private_target_name (priv));
+
+    switch (priv->state)
+    {
+    case CLIP_STATE_IDLE:
+        /* We're idle, store right now and mark us BUSY for 10 secs */
+        gtk_clipboard_store (priv->real);
+        priv->state = CLIP_STATE_BUSY;
+        g_timeout_add_seconds (10, g_paste_clipboard_private_store_timeout, priv);
+        break;
+    case CLIP_STATE_BUSY:
+        /* We're busy, wait until we're finished to store once more */
+        priv->state = CLIP_STATE_WAITING;
+        break;
+    case CLIP_STATE_WAITING:
+        /* We're already waiting, nothing to do */
+        break;
+    }
+}
+
 /**
  * g_paste_clipboard_select_text:
  * @self: a #GPasteClipboard instance
@@ -285,14 +356,13 @@ g_paste_clipboard_select_text (GPasteClipboard *self,
     g_return_if_fail (g_utf8_validate (text, -1, NULL));
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
-    GtkClipboard *real = priv->real;
 
-    /* Avoid cycling twice */
+    g_debug("%s: select text", _g_paste_clipboard_private_target_name (priv));
+
+    /* Avoid cycling twice as gtk_clipboard_set_text will make the clipboards manager react */
     g_paste_clipboard_private_set_text (priv, text);
-
-    /* Let the clipboards manager handle our internal text */
-    gtk_clipboard_set_text (real, text, -1);
-    gtk_clipboard_store (real);
+    gtk_clipboard_set_text (priv->real, text, -1);
+    g_paste_clipboard_private_store (priv);
 }
 
 static void
@@ -357,6 +427,8 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
     GtkClipboard *real = priv->real;
     g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
 
+    g_debug("%s: select uris", _g_paste_clipboard_private_target_name (priv));
+
     g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (G_PASTE_ITEM (item)));
 
     gtk_target_list_add_text_targets (target_list, 0);
@@ -371,7 +443,7 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
                                   g_paste_clipboard_get_clipboard_data,
                                   g_paste_clipboard_clear_clipboard_data,
                                   g_object_ref (item));
-    gtk_clipboard_store (real);
+    g_paste_clipboard_private_store (priv);
 
     gtk_target_table_free (targets, n_targets);
 }
@@ -391,6 +463,12 @@ g_paste_clipboard_clear (GPasteClipboard *self)
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
+    /* We're already clear, don't enter an infinite event loop */
+    if (!priv->text && !priv->image_checksum)
+        return;
+
+    g_debug("%s: clear", _g_paste_clipboard_private_target_name (priv));
+
     g_clear_pointer (&priv->text, g_free);
     g_clear_pointer (&priv->image_checksum, g_free);
 
@@ -435,6 +513,8 @@ g_paste_clipboard_private_select_image (GPasteClipboardPrivate *priv,
 
     GtkClipboard *real = priv->real;
 
+    g_debug("%s: select image", _g_paste_clipboard_private_target_name (priv));
+
     g_paste_clipboard_private_set_image_checksum (priv, checksum);
     gtk_clipboard_set_image (real, image);
 }
@@ -517,28 +597,29 @@ g_paste_clipboard_set_image (GPasteClipboard             *self,
  *
  * Put the value of the item into the #GPasteClipbaord and the intern GtkClipboard
  *
- * Returns:
+ * Returns: %FALSE if the item was invalid, %TRUE otherwise
  */
-G_PASTE_VISIBLE void
+G_PASTE_VISIBLE gboolean
 g_paste_clipboard_select_item (GPasteClipboard  *self,
                                const GPasteItem *item)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
-    g_return_if_fail (G_PASTE_IS_ITEM (item));
+    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), FALSE);
+    g_return_val_if_fail (G_PASTE_IS_ITEM (item), FALSE);
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
+    g_debug("%s: select item", _g_paste_clipboard_private_target_name (priv));
+
     if (G_PASTE_IS_IMAGE_ITEM (item))
     {
         GPasteImageItem *image_item = G_PASTE_IMAGE_ITEM (item);
         const gchar *checksum = g_paste_image_item_get_checksum (image_item);
+        GdkPixbuf *image = g_paste_image_item_get_image (image_item);
 
-        if (g_strcmp0 (checksum, priv->image_checksum))
-        {
-            g_paste_clipboard_private_select_image (priv,
-                                                    g_paste_image_item_get_image (image_item),
-                                                    checksum);
-        }
+        if (!image || !GDK_IS_PIXBUF (image))
+            return FALSE;
+        else if (g_strcmp0 (checksum, priv->image_checksum))
+            g_paste_clipboard_private_select_image (priv, image, checksum);
     }
     else
     {
@@ -554,6 +635,8 @@ g_paste_clipboard_select_item (GPasteClipboard  *self,
                 g_assert_not_reached ();
         }
     }
+
+    return TRUE;
 }
 
 /**
@@ -566,8 +649,8 @@ g_paste_clipboard_select_item (GPasteClipboard  *self,
  * Returns:
  */
 G_PASTE_VISIBLE void
-g_paste_clipboard_ensure_not_empty (GPasteClipboard     *self,
-                                    const GPasteHistory *history)
+g_paste_clipboard_ensure_not_empty (GPasteClipboard *self,
+                                    GPasteHistory   *history)
 {
     g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
     g_return_if_fail (G_PASTE_IS_HISTORY (history));
@@ -575,7 +658,12 @@ g_paste_clipboard_ensure_not_empty (GPasteClipboard     *self,
     const GSList *hist = g_paste_history_get_history (history);
 
     if (hist)
-        g_paste_clipboard_select_item (self, hist->data);
+    {
+        GPasteItem *item = hist->data;
+
+        if (!g_paste_clipboard_select_item (self, item))
+            g_paste_history_remove (history, 0);
+    }
 }
 
 static void
@@ -584,6 +672,9 @@ g_paste_clipboard_owner_change (GtkClipboard        *clipboard G_GNUC_UNUSED,
                                 gpointer             user_data)
 {
     GPasteClipboard *self = user_data;
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    g_debug("%s: owner change", _g_paste_clipboard_private_target_name (priv));
 
     g_signal_emit (self,
 		   signals[OWNER_CHANGE],
@@ -696,6 +787,8 @@ g_paste_clipboard_init (GPasteClipboard *self)
 
     priv->text = NULL;
     priv->image_checksum = NULL;
+
+    priv->state = CLIP_STATE_IDLE;
 }
 
 /**
