@@ -96,6 +96,10 @@ g_paste_clipboards_manager_sync_from_to (GPasteClipboardsManager *self,
 
     GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
     GtkClipboard *_from = NULL, *_to = NULL;
+    g_autofree gchar *_from_name = gdk_atom_name (from);
+    g_autofree gchar *_to_name = gdk_atom_name (to);
+
+    g_debug ("clipboards-manager: sync from %s to %s", _from_name, _to_name);
 
     for (GSList *clipboard = priv->clipboards; clipboard; clipboard = g_slist_next (clipboard))
     {
@@ -126,6 +130,8 @@ g_paste_clipboards_manager_notify_finish (GPasteClipboardsManagerPrivate *priv,
 {
     GPasteHistory *history = priv->history;
 
+    g_debug ("clipboards-manager: notify finish");
+
     if (item)
         g_paste_history_add (history, item);
 
@@ -167,6 +173,8 @@ g_paste_clipboards_manager_text_ready (GPasteClipboard *clipboard,
     GPasteItem *item = NULL;
     const gchar *synchronized_text = NULL;
 
+    g_debug ("clipboards-manager: text ready");
+
     /* Did we already have some contents, or did we get some now? */
     gboolean something_in_clipboard = !!g_paste_clipboard_get_text (clipboard);
 
@@ -197,6 +205,8 @@ g_paste_clipboards_manager_image_ready (GPasteClipboard *clipboard,
     GPasteClipboardsManagerPrivate *priv = data->priv;
     GPasteItem *item = NULL;
 
+    g_debug ("clipboards-manager: image ready");
+
     /* Did we already have some contents, or did we get some now? */
     gboolean something_in_clipboard = !!g_paste_clipboard_get_image_checksum (clipboard);
 
@@ -214,6 +224,8 @@ g_paste_clipboards_manager_targets_ready (GtkClipboard     *clipboard G_GNUC_UNU
 {
     g_autofree GPasteClipboardsManagerCallbackData *data = user_data;
 
+    g_debug ("clipboards-manager: targets ready");
+
     if (gtk_selection_data_get_length (targets) >= 0)
     {
         data->uris_available = gtk_selection_data_targets_include_uri (targets);
@@ -237,6 +249,8 @@ g_paste_clipboards_manager_targets_ready (GtkClipboard     *clipboard G_GNUC_UNU
     }
     else
     {
+        g_debug ("clipboards-manager: no target ready");
+
         g_paste_clipboard_clear (data->clip);
         g_paste_clipboard_ensure_not_empty (data->clip, data->priv->history);
     }
@@ -249,6 +263,8 @@ g_paste_clipboards_manager_notify (GPasteClipboard *clipboard,
 {
     GPasteClipboardsManagerPrivate *priv = user_data;
 
+    g_debug ("clipboards-manager: notify");
+
     GPasteSettings *settings = priv->settings;
     GdkAtom atom = g_paste_clipboard_get_target (clipboard);
     gboolean track = ((atom != GDK_SELECTION_PRIMARY || g_paste_settings_get_primary_to_history (settings)) &&
@@ -299,23 +315,31 @@ g_paste_clipboards_manager_activate (GPasteClipboardsManager *self)
  *
  * Select a new #GPasteItem
  *
- * Returns:
+ * Returns: %FALSE if the item was invalid, %TRUE otherwise
  */
-G_PASTE_VISIBLE void
+G_PASTE_VISIBLE gboolean
 g_paste_clipboards_manager_select (GPasteClipboardsManager *self,
                                    const GPasteItem        *item)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARDS_MANAGER (self));
-    g_return_if_fail (G_PASTE_IS_ITEM (item));
+    g_return_val_if_fail (G_PASTE_IS_CLIPBOARDS_MANAGER (self), FALSE);
+    g_return_val_if_fail (G_PASTE_IS_ITEM (item), FALSE);
 
     GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
 
+    g_debug ("clipboards-manager: select");
+
     for (GSList *clipboard = priv->clipboards; clipboard; clipboard = g_slist_next (clipboard))
     {
         _Clipboard *clip = clipboard->data;
 
-        g_paste_clipboard_select_item (clip->clipboard, item);
+        if (!g_paste_clipboard_select_item (clip->clipboard, item))
+        {
+            g_debug ("clipboards-manager: item was invalid, deleting it");
+            return FALSE;
+        }
     }
+
+    return TRUE;
 }
 
 static void
@@ -323,7 +347,10 @@ on_item_selected (GPasteClipboardsManager *self,
                   GPasteItem              *item,
                   GPasteHistory           *history G_GNUC_UNUSED)
 {
-    g_paste_clipboards_manager_select (self, item);
+    GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (G_PASTE_CLIPBOARDS_MANAGER (self));
+
+    if (!g_paste_clipboards_manager_select (self, item))
+        g_paste_history_remove (priv->history, 0);
 }
 
 static void
