@@ -32,14 +32,14 @@ G_BEGIN_DECLS
 
 G_PASTE_FINAL_TYPE (ClipboardsManager, clipboards_manager, CLIPBOARDS_MANAGER, GObject)
 
-void g_paste_clipboards_manager_add_clipboard (GPasteClipboardsManager *self,
-                                               GPasteClipboard         *clipboard);
-void g_paste_clipboards_manager_sync_from_to  (GPasteClipboardsManager *self,
-                                               GdkAtom                  from,
-                                               GdkAtom                  to);
-void g_paste_clipboards_manager_activate      (GPasteClipboardsManager *self);
-void g_paste_clipboards_manager_select        (GPasteClipboardsManager *self,
-                                               const GPasteItem        *item);
+void     g_paste_clipboards_manager_add_clipboard (GPasteClipboardsManager *self,
+                                                   GPasteClipboard         *clipboard);
+void     g_paste_clipboards_manager_sync_from_to  (GPasteClipboardsManager *self,
+                                                   GdkAtom                  from,
+                                                   GdkAtom                  to);
+void     g_paste_clipboards_manager_activate      (GPasteClipboardsManager *self);
+gboolean g_paste_clipboards_manager_select        (GPasteClipboardsManager *self,
+                                                   const GPasteItem        *item);
 
 GPasteClipboardsManager *g_paste_clipboards_manager_new (GPasteHistory  *history,
                                                          GPasteSettings *settings);
