@@ -60,6 +60,8 @@ static guint64 signals[LAST_SIGNAL] = { 0 };
 static void
 g_paste_history_private_elect_new_biggest (GPasteHistoryPrivate *priv)
 {
+    g_debug ("history: elect biggest");
+
     priv->biggest_index = 0;
     priv->biggest_size = 0;
 
@@ -113,6 +115,8 @@ static void
 g_paste_history_selected (GPasteHistory *self,
                           GPasteItem    *item)
 {
+    g_debug ("history: selected");
+
     g_signal_emit (self,
                    signals[SELECTED],
                    0, /* detail */
@@ -124,6 +128,8 @@ static void
 g_paste_history_emit_switch (GPasteHistory *self,
                              const gchar   *name)
 {
+    g_debug ("history: switch");
+
     g_signal_emit (self,
                    signals[SWITCH],
                    0, /* detail */
@@ -139,6 +145,8 @@ g_paste_history_update (GPasteHistory     *self,
 {
     g_paste_history_save (self, NULL);
 
+    g_debug ("history: update");
+
     g_signal_emit (self,
                    signals[UPDATE],
                    0, /* detail */
@@ -244,6 +252,8 @@ g_paste_history_add (GPasteHistory *self,
     gboolean election_needed = FALSE;
     GPasteUpdateTarget target = G_PASTE_UPDATE_TARGET_ALL;
 
+    g_debug ("history: add");
+
     if (history)
     {
         GPasteItem *old_first = history->data;
@@ -321,6 +331,8 @@ g_paste_history_remove (GPasteHistory *self,
     GPasteHistoryPrivate *priv = g_paste_history_get_instance_private (self);
     GSList *history = priv->history;
 
+    g_debug ("history: remove '%" G_GUINT64_FORMAT "'", pos);
+
     g_return_if_fail (pos < g_slist_length (history));
 
     if (pos)
@@ -451,6 +463,8 @@ g_paste_history_select (GPasteHistory *self,
 {
     g_return_if_fail (G_PASTE_IS_HISTORY (self));
 
+    g_debug ("history: select '%" G_GUINT64_FORMAT "'", index);
+
     GPasteHistoryPrivate *priv = g_paste_history_get_instance_private (self);
     GSList *history = priv->history;
 
@@ -1199,6 +1213,8 @@ g_paste_history_history_name_changed (GPasteHistory *self)
 {
     GPasteHistoryPrivate *priv = g_paste_history_get_instance_private (self);
 
+    g_debug ("history: name cahnged to '%s'", priv->name);
+
     g_paste_history_load (self, NULL);
     g_paste_history_emit_switch (self, priv->name);
     g_paste_history_update (self, G_PASTE_UPDATE_ACTION_REPLACE, G_PASTE_UPDATE_TARGET_ALL, 0);
@@ -1212,6 +1228,7 @@ g_paste_history_settings_changed (GPasteSettings *settings G_GNUC_UNUSED,
     GPasteHistory *self = user_data;
     GPasteHistoryPrivate *priv = g_paste_history_get_instance_private (self);
 
+    /* FIXME: track text item size settings */
     if (!g_strcmp0(key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
         g_paste_history_private_check_size (priv);
     else if (!g_strcmp0 (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
@@ -1399,6 +1416,8 @@ g_paste_history_search (const GPasteHistory *self,
     g_return_val_if_fail (G_PASTE_IS_HISTORY (self), NULL);
     g_return_val_if_fail (pattern && g_utf8_validate (pattern, -1, NULL), NULL);
 
+    g_debug ("history: search '%s'", pattern);
+
     GPasteHistoryPrivate *priv = g_paste_history_get_instance_private (self);
     g_autoptr (GError) error = NULL;
     g_autoptr (GRegex) regex = g_regex_new (pattern,
