@@ -226,7 +226,8 @@ g_paste_daemon_private_do_add_item (GPasteDaemonPrivate *priv,
                                     GPasteItem          *item)
 {
     g_paste_history_add (priv->history, item);
-    g_paste_clipboards_manager_select (priv->clipboards_manager, item);
+    if (!g_paste_clipboards_manager_select (priv->clipboards_manager, item))
+        g_paste_history_remove (priv->history, 0);
 }
 
 static void
@@ -664,10 +665,14 @@ g_paste_daemon_private_rename_password (GPasteDaemonPrivate *priv,
 
 static GVariant *
 g_paste_daemon_private_search (GPasteDaemonPrivate *priv,
-                               GVariant            *parameters)
+                               GVariant                  *parameters,
+                               GPasteDBusError          **err)
 {
     g_autofree gchar *name = g_paste_daemon_get_dbus_string_parameter (parameters, NULL);
     g_autoptr (GArray) results = g_paste_history_search (priv->history, name);
+
+    G_PASTE_DBUS_ASSERT_FULL (results, "Error while performing search", NULL);
+
     GVariant *variant = g_variant_new_fixed_array (G_VARIANT_TYPE_UINT64, results->data, results->len, sizeof (guint64));
 
     return g_variant_new_tuple (&variant, 1);
@@ -890,7 +895,7 @@ g_paste_daemon_dbus_method_call (GDBusConnection       *connection     G_GNUC_UN
     else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_REPLACE))
         g_paste_daemon_private_replace (priv, parameters, &err);
     else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SEARCH))
-        answer = g_paste_daemon_private_search (priv, parameters);
+        answer = g_paste_daemon_private_search (priv, parameters, &err);
     else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SELECT))
         g_paste_daemon_private_select (priv, parameters);
     else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SET_PASSWORD))
@@ -978,6 +983,7 @@ g_paste_daemon_on_screensaver_active_changed (GPasteDaemonPrivate *priv,
     /* We always do the activate action, so that the deactivate one works anyways */
     {
         g_autoptr (GPasteItem) item = g_paste_text_item_new ("");
+        /* will always return TRUE */
         g_paste_clipboards_manager_select (priv->clipboards_manager, item);
     }
 
@@ -986,7 +992,10 @@ g_paste_daemon_on_screensaver_active_changed (GPasteDaemonPrivate *priv,
         const GPasteItem *item = g_paste_history_get (priv->history, 0);
 
         if (item)
-            g_paste_clipboards_manager_select (priv->clipboards_manager, item);
+        {
+            if (!g_paste_clipboards_manager_select (priv->clipboards_manager, item))
+                g_paste_history_remove (priv->history, 0);
+        }
     }
 }
 
