@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2011-2013 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,9 +17,7 @@
  *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gpaste-keybinder-private.h"
-
-#include <gdk/gdk.h>
+#include <gpaste-keybinder.h>
 
 #ifdef GDK_WINDOWING_WAYLAND
 #  include <gdk/gdkwayland.h>
@@ -31,7 +29,12 @@
 
 #define MAX_BINDINGS 7
 
-struct _GPasteKeybinderPrivate
+struct _GPasteKeybinder
+{
+    GObject parent_instance;
+};
+
+typedef struct
 {
     GSList                 *keybindings;
 
@@ -39,11 +42,11 @@ struct _GPasteKeybinderPrivate
     GPasteGnomeShellClient *shell_client;
     gboolean                use_shell_client;
     gboolean                grabbing;
-    guint                   retries;
+    guint64                 retries;
 
     gulong                  accel_signal;
-    guint                   shell_watch;
-};
+    guint64                 shell_watch;
+} GPasteKeybinderPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GPasteKeybinder, g_paste_keybinder, G_TYPE_OBJECT)
 
@@ -55,7 +58,7 @@ G_DEFINE_TYPE_WITH_PRIVATE (GPasteKeybinder, g_paste_keybinder, G_TYPE_OBJECT)
 static void
 g_paste_keybinder_change_grab_wayland (void)
 {
-    g_critical ("Wayland is currently not supported outside of gnome-shell.");
+    g_warning ("Wayland hotkeys are currently not supported outside of gnome-shell.");
 }
 #endif
 
@@ -75,7 +78,7 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
 
     gdk_error_trap_push ();
 
-    guint mod_masks [] = {
+    guint64 mod_masks [] = {
         0, /* modifier only */
         GDK_MOD2_MASK, /* NumLock */
         GDK_MOD5_MASK, /* ScrollLock */
@@ -88,11 +91,11 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
 
     Window window = GDK_ROOT_WINDOW ();
     GdkModifierType modifiers = g_paste_keybinding_get_modifiers (binding);
-    const guint *keycodes = g_paste_keybinding_get_keycodes (binding);
+    const guint32 *keycodes = g_paste_keybinding_get_keycodes (binding);
 
-    for (guint i = 0; i < G_N_ELEMENTS (mod_masks); ++i) {
+    for (guint64 i = 0; i < G_N_ELEMENTS (mod_masks); ++i) {
         XIGrabModifiers mods = { mod_masks[i] | modifiers, 0 };
-        for (const guint *keycode = keycodes; *keycode; ++keycode)
+        for (const guint32 *keycode = keycodes; *keycode; ++keycode)
         {
             if (grab)
             {
@@ -173,6 +176,20 @@ _keybinding_deactivate (_Keybinding *k)
         g_paste_keybinding_deactivate (k->binding);
 }
 
+static void
+on_key_grabbed (GObject      *source_object,
+                GAsyncResult *res,
+                gpointer      user_data)
+{
+    _Keybinding *k = user_data;
+    g_autoptr (GError) error = NULL;
+
+    k->action = g_paste_gnome_shell_client_grab_accelerator_finish (G_PASTE_GNOME_SHELL_CLIENT (source_object), res, &error);
+
+    if (error)
+        g_warning ("Couldn't grab keybinding: %s", error->message);
+}
+
 static void
 _keybinding_grab_gnome_shell (_Keybinding *k)
 {
@@ -181,14 +198,10 @@ _keybinding_grab_gnome_shell (_Keybinding *k)
 
     GPasteGnomeShellAccelerator accel = {
         g_paste_keybinding_get_accelerator (k->binding, k->settings),
-        G_PASTE_GNOME_SHELL_KEYBINDING_MODE_ALL
+        G_PASTE_GNOME_SHELL_ACTION_MODE_ALL
     };
 
-    G_PASTE_CLEANUP_ERROR_FREE GError *error = NULL;
-    k->action = g_paste_gnome_shell_client_grab_accelerator_sync (k->shell_client, accel, &error);
-
-    if (error)
-        g_warning ("Couldn't grab keybinding: %s", error->message);
+    g_paste_gnome_shell_client_grab_accelerator (k->shell_client, accel, on_key_grabbed, k);
 }
 
 static void
@@ -202,19 +215,28 @@ _keybinding_grab (_Keybinding *k)
 }
 
 static void
-_keybinding_ungrab_gnome_shell (_Keybinding *k)
+on_key_ungrabbed (GObject      *source_object,
+                  GAsyncResult *res,
+                  gpointer      user_data G_GNUC_UNUSED)
 {
-    G_PASTE_CLEANUP_ERROR_FREE GError *error = NULL;
-    if (k->action)
-    {
-        g_paste_gnome_shell_client_ungrab_accelerator_sync (k->shell_client, k->action, &error);
-        k->action = 0;
-    }
+    g_autoptr (GError) error = NULL;
+
+    g_paste_gnome_shell_client_ungrab_accelerator_finish (G_PASTE_GNOME_SHELL_CLIENT (source_object), res, &error);
 
     if (error)
         g_warning ("Couldn't ungrab keybinding: %s", error->message);
 }
 
+static void
+_keybinding_ungrab_gnome_shell (_Keybinding *k)
+{
+    if (k->action)
+    {
+        g_paste_gnome_shell_client_ungrab_accelerator (k->shell_client, k->action, on_key_ungrabbed, NULL);
+        k->action = 0;
+    }
+}
+
 static void
 _keybinding_ungrab (_Keybinding *k)
 {
@@ -248,7 +270,7 @@ _keybinding_new (GPasteKeybinding       *binding,
 
     k->action = 0;
 
-    G_PASTE_CLEANUP_FREE gchar *detailed_signal = g_strdup_printf ("rebind::%s",
+    g_autofree gchar *detailed_signal = g_strdup_printf ("rebind::%s",
                                                                    g_paste_keybinding_get_dconf_key (binding));
     k->rebind_signal = g_signal_connect_swapped (settings,
                                                  detailed_signal,
@@ -339,10 +361,8 @@ grab_accelerators_cb (GObject      *source_object,
                       gpointer      user_data)
 {
     GPasteKeybinderPrivate *priv = user_data;
-    G_PASTE_CLEANUP_ERROR_FREE GError *error = NULL;
-    G_PASTE_CLEANUP_FREE guint *actions = g_paste_gnome_shell_client_grab_accelerators_finish (G_PASTE_GNOME_SHELL_CLIENT (source_object),
-                                                                                               res,
-                                                                                               &error);
+    g_autoptr (GError) error = NULL;
+    g_autofree guint32 *actions = g_paste_gnome_shell_client_grab_accelerators_finish (G_PASTE_GNOME_SHELL_CLIENT (source_object), res, &error);
 
     if (error)
     {
@@ -361,7 +381,7 @@ grab_accelerators_cb (GObject      *source_object,
     {
         priv->retries = 0;
 
-        guint index = 0;
+        guint64 index = 0;
         for (GSList *binding = priv->keybindings; binding; binding = g_slist_next (binding))
         {
             _Keybinding *k = binding->data;
@@ -384,7 +404,7 @@ g_paste_keybinder_private_grab_all_gnome_shell (GPasteKeybinderPrivate *priv)
     GPasteGnomeShellAccelerator accels[MAX_BINDINGS + 1];
     GPasteSettings *settings = priv->settings;
 
-    guint index = 0;
+    guint64 index = 0;
     for (GSList *binding = priv->keybindings; binding && index < MAX_BINDINGS; binding = g_slist_next (binding))
     {
         _Keybinding *k = binding->data;
@@ -462,7 +482,7 @@ g_paste_keybinder_deactivate_all (GPasteKeybinder *self)
 static void
 g_paste_keybinder_parse_event_wayland (void)
 {
-    g_critical ("Wayland is currently not supported outside of gnome-shell.");
+    g_warning ("Wayland hotkeys are currently not supported outside of gnome-shell.");
 }
 #endif
 
@@ -470,13 +490,13 @@ g_paste_keybinder_parse_event_wayland (void)
 static gint
 g_paste_keybinder_get_xinput_opcode (Display *display)
 {
-    static gint xinput_opcode = 0;
+    static gint32 xinput_opcode = 0;
 
     if (!xinput_opcode)
     {
-        gint major = 2, minor = 3;
-        gint xinput_error_base;
-        gint xinput_event_base;
+        gint32 major = 2, minor = 3;
+        gint32 xinput_error_base;
+        gint32 xinput_event_base;
 
         if (XQueryExtension (display,
                              "XInputExtension",
@@ -495,7 +515,7 @@ g_paste_keybinder_get_xinput_opcode (Display *display)
 static void
 g_paste_keybinder_parse_event_x11 (XEvent                  *event,
                                    GdkModifierType         *modifiers,
-                                   guint                   *keycode)
+                                   guint64                 *keycode)
 {
     XGenericEventCookie cookie = event->xcookie;
 
@@ -532,7 +552,7 @@ g_paste_keybinder_filter (GdkXEvent *xevent,
 
     GdkDisplay *display = gdk_display_get_default ();
     GdkModifierType modifiers = 0;
-    guint keycode = 0;
+    guint64 keycode = 0;
 
 #ifdef GDK_WINDOWING_WAYLAND
     if (GDK_IS_WAYLAND_DISPLAY (display))
@@ -559,8 +579,6 @@ g_paste_keybinder_filter (GdkXEvent *xevent,
 static void
 on_accelerator_activated (GPasteGnomeShellClient *client    G_GNUC_UNUSED,
                           guint32                 action,
-                          guint32                 deviceid  G_GNUC_UNUSED,
-                          guint32                 timestamp G_GNUC_UNUSED,
                           gpointer                user_data)
 {
     GPasteKeybinderPrivate *priv = user_data;
