@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2011-2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,25 +17,30 @@
  *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gpaste-settings-private.h"
+#include <gpaste-gsettings-keys.h>
+#include <gpaste-settings.h>
 
-#include <gio/gio.h>
+struct _GPasteSettings
+{
+    GObject parent_instance;
+};
 
-struct _GPasteSettingsPrivate
+typedef struct
 {
     GSettings *settings;
     GSettings *shell_settings;
 
-    guint32    element_size;
+    guint64    element_size;
     gboolean   growing_lines;
     gchar     *history_name;
     gboolean   images_support;
+    gchar     *launch_ui;
     gchar     *make_password;
-    guint32    max_displayed_history_size;
-    guint32    max_history_size;
-    guint32    max_memory_usage;
-    guint32    max_text_item_size;
-    guint32    min_text_item_size;
+    guint64    max_displayed_history_size;
+    guint64    max_history_size;
+    guint64    max_memory_usage;
+    guint64    max_text_item_size;
+    guint64    min_text_item_size;
     gchar     *pop;
     gboolean   primary_to_history;
     gboolean   save_history;
@@ -46,12 +51,13 @@ struct _GPasteSettingsPrivate
     gboolean   track_changes;
     gboolean   track_extension_state;
     gboolean   trim_items;
+    gchar     *upload;
 
     gboolean   extension_enabled;
 
     gulong     changed_signal;
     gulong     shell_changed_signal;
-};
+} GPasteSettingsPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettings, g_paste_settings, G_TYPE_OBJECT)
 
@@ -64,21 +70,21 @@ enum
     LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint64 signals[LAST_SIGNAL] = { 0 };
 
 #define SETTING(name, key, type, setting_type, fail, guards, clear_func, dup_func)                     \
     G_PASTE_VISIBLE type                                                                               \
     g_paste_settings_get_##name (const GPasteSettings *self)                                           \
     {                                                                                                  \
-        g_return_val_if_fail (G_PASTE_IS_SETTINGS (self), fail);                                       \
-        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private ((GPasteSettings *) self); \
+        g_return_val_if_fail (G_PASTE_IS_SETTINGS ((gpointer) self), fail);                                       \
+        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self); \
         return priv->name;                                                                             \
     }                                                                                                  \
     G_PASTE_VISIBLE void                                                                               \
     g_paste_settings_reset_##name (GPasteSettings *self)                                               \
     {                                                                                                  \
         g_return_if_fail (G_PASTE_IS_SETTINGS (self));                                                 \
-        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private ((GPasteSettings *) self); \
+        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self); \
         g_settings_reset (priv->settings, G_PASTE_##key##_SETTING);                                    \
     }                                                                                                  \
     static void                                                                                        \
@@ -102,7 +108,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
     SETTING (name, key, type, setting_type, fail, {}, {},)
 
 #define BOOLEAN_SETTING(name, key) TRIVIAL_SETTING (name, key, gboolean, boolean, FALSE)
-#define UNSIGNED_SETTING(name, key) TRIVIAL_SETTING (name, key, guint32, uint, 0)
+#define UNSIGNED_SETTING(name, key) TRIVIAL_SETTING (name, key, guint64, gpaste_uint64, 0)
 
 #define STRING_SETTING(name, key) SETTING (name, key, const gchar *, string, NULL,                \
                                            g_return_if_fail (value);                              \
@@ -124,6 +130,23 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define NEW_SIGNAL_DETAILED(name, arg_type) NEW_SIGNAL_FULL (name, G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, arg_type, arg_type)
 #define NEW_SIGNAL_DETAILED_STATIC(name, arg_type) NEW_SIGNAL_FULL (name, G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, arg_type, arg_type | G_SIGNAL_TYPE_STATIC_SCOPE)
 
+static guint64
+g_settings_get_gpaste_uint64 (GSettings   *settings,
+                              const gchar *key)
+{
+    g_autoptr (GVariant) v = g_settings_get_value (settings, key);
+
+    return g_variant_get_uint64 (v);
+}
+
+static void
+g_settings_set_gpaste_uint64 (GSettings   *settings,
+                              const gchar *key,
+                              guint64      value)
+{
+    g_settings_set_value (settings, key, g_variant_new_uint64 (value));
+}
+
 /**
  * g_paste_settings_get_element_size:
  * @self: a #GPasteSettings instance
@@ -232,6 +255,33 @@ STRING_SETTING (history_name, HISTORY_NAME)
  */
 BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
 
+/**
+ * g_paste_settings_get_launch_ui:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "launch-ui" setting
+ *
+ * Returns: the value of the "launch-ui" setting
+ */
+/**
+ * g_paste_settings_reset_launch_ui:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "launch-ui" setting
+ *
+ * Returns:
+ */
+/**
+ * g_paste_settings_set_launch_ui:
+ * @self: a #GPasteSettings instance
+ * @value: the new keyboard shortcut
+ *
+ * Change the "launch-ui" setting
+ *
+ * Returns:
+ */
+STRING_SETTING (launch_ui, LAUNCH_UI)
+
 /**
  * g_paste_settings_get_make_password:
  * @self: a #GPasteSettings instance
@@ -664,7 +714,34 @@ BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
  */
 BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
 
-#if G_PASTE_CONFIG_ENABLE_EXTENSION
+/**
+ * g_paste_settings_get_upload:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "upload" setting
+ *
+ * Returns: the value of the "upload" setting
+ */
+/**
+ * g_paste_settings_reset_upload:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "upload" setting
+ *
+ * Returns:
+ */
+/**
+ * g_paste_settings_set_upload:
+ * @self: a #GPasteSettings instance
+ * @value: the new keyboard shortcut
+ *
+ * Change the "upload" setting
+ *
+ * Returns:
+ */
+STRING_SETTING (upload, UPLOAD)
+
+#ifdef ENABLE_EXTENSION
 #define EXTENSION_NAME "GPaste@gnome-shell-extensions.gnome.org"
 /**
  * g_paste_settings_get_extension_enabled:
@@ -677,8 +754,8 @@ BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
 G_PASTE_VISIBLE gboolean
 g_paste_settings_get_extension_enabled (const GPasteSettings *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS (self), FALSE);
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private ((GPasteSettings *) self);
+    g_return_val_if_fail (G_PASTE_IS_SETTINGS ((gpointer) self), FALSE);
+    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
     return priv->extension_enabled;
 }
 
@@ -691,8 +768,8 @@ g_paste_settings_private_get_enabled_extensions (GPasteSettingsPrivate *priv)
 static void
 g_paste_settings_private_set_extension_enabled_from_dconf (GPasteSettingsPrivate *priv)
 {
-    G_PASTE_CLEANUP_STRFREEV gchar **extensions = g_paste_settings_private_get_enabled_extensions (priv);
-    for (gchar **e = extensions; *e; ++e)
+    g_auto (GStrv) extensions = g_paste_settings_private_get_enabled_extensions (priv);
+    for (GStrv e = extensions; *e; ++e)
     {
         if (!g_strcmp0 (*e, EXTENSION_NAME))
         {
@@ -717,13 +794,13 @@ g_paste_settings_set_extension_enabled (GPasteSettings *self,
                                         gboolean        value)
 {
     g_return_if_fail (G_PASTE_IS_SETTINGS (self));
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private ((GPasteSettings *) self);
-    G_PASTE_CLEANUP_STRFREEV gchar **extensions = NULL;
+    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
+    g_auto (GStrv) extensions = NULL;
     if (!priv->shell_settings || (value == priv->extension_enabled))
         return;
 
     extensions = g_paste_settings_private_get_enabled_extensions (priv);
-    gsize nb = g_strv_length (extensions);
+    guint64 nb = g_strv_length (extensions);
     if (value)
     {
         extensions = g_realloc (extensions, (nb + 2) * sizeof (gchar *));
@@ -733,7 +810,7 @@ g_paste_settings_set_extension_enabled (GPasteSettings *self,
     else
     {
         gboolean found = FALSE;
-        for (gsize i = 0; i < nb; ++i)
+        for (guint64 i = 0; i < nb; ++i)
         {
             if (!found && !g_strcmp0 (extensions[i], EXTENSION_NAME))
             {
@@ -794,6 +871,11 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
         g_paste_settings_private_set_history_name_from_dconf (priv);
     else if (!g_strcmp0 (key, G_PASTE_IMAGES_SUPPORT_SETTING))
         g_paste_settings_private_set_images_support_from_dconf (priv);
+    else if (!g_strcmp0 (key, G_PASTE_LAUNCH_UI_SETTING))
+    {
+        g_paste_settings_private_set_launch_ui_from_dconf (priv);
+        g_paste_settings_rebind (self, G_PASTE_LAUNCH_UI_SETTING);
+    }
     else if (!g_strcmp0 (key, G_PASTE_MAKE_PASSWORD_SETTING))
     {
         g_paste_settings_private_set_make_password_from_dconf (priv);
@@ -848,6 +930,11 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
         g_paste_settings_private_set_track_extension_state_from_dconf (priv);
     else if (!g_strcmp0 (key, G_PASTE_TRIM_ITEMS_SETTING))
         g_paste_settings_private_set_trim_items_from_dconf (priv);
+    else if (!g_strcmp0 (key, G_PASTE_UPLOAD_SETTING))
+    {
+        g_paste_settings_private_set_upload_from_dconf (priv);
+        g_paste_settings_rebind (self, G_PASTE_UPLOAD_SETTING);
+    }
 
     /* Forward the signal */
     g_signal_emit (self,
@@ -869,7 +956,7 @@ g_paste_settings_dispose (GObject *object)
         g_clear_object (&priv->settings);
     }
 
-#if G_PASTE_CONFIG_ENABLE_EXTENSION
+#ifdef ENABLE_EXTENSION
     GSettings *shell_settings = priv->shell_settings;
 
     if (shell_settings)
@@ -888,11 +975,13 @@ g_paste_settings_finalize (GObject *object)
     GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
 
     g_free (priv->history_name);
+    g_free (priv->launch_ui);
     g_free (priv->make_password);
     g_free (priv->pop);
     g_free (priv->show_history);
     g_free (priv->sync_clipboard_to_primary);
     g_free (priv->sync_primary_to_clipboard);
+    g_free (priv->upload);
 
     G_OBJECT_CLASS (g_paste_settings_parent_class)->finalize (object);
 }
@@ -905,8 +994,44 @@ g_paste_settings_class_init (GPasteSettingsClass *klass)
     object_class->dispose = g_paste_settings_dispose;
     object_class->finalize = g_paste_settings_finalize;
 
+    /**
+     * GPasteSettings::changed:
+     * @settings: the object on which the signal was emitted
+     * @key: the name of the key that changed
+     *
+     * The "changed" signal is emitted when a key has potentially changed.
+     * You should call one of the g_paste_settings_get() calls to check the new
+     * value.
+     *
+     * This signal supports detailed connections.  You can connect to the
+     * detailed signal "changed::x" in order to only receive callbacks
+     * when key "x" changes.
+     */
     signals[CHANGED] = NEW_SIGNAL_DETAILED_STATIC ("changed", STRING);
+
+    /**
+     * GPasteSettings::rebind:
+     * @settings: the object on which the signal was emitted
+     * @key: the name of the key that changed
+     *
+     * The "rebind" signal is emitted when a key has potentially changed.
+     * You should call one of the g_paste_settings_get() calls to check the new
+     * value.
+     *
+     * This signal supports detailed connections.  You can connect to the
+     * detailed signal "rebind::x" in order to only receive callbacks
+     * when key "x" changes.
+     */
     signals[REBIND]  = NEW_SIGNAL_DETAILED        ("rebind" , STRING);
+
+    /**
+     * GPasteSettings::track:
+     * @settings: the object on which the signal was emitted
+     * @tracking_state: whether we're now tracking or not
+     *
+     * The "track" signal is emitted when the daemon starts or stops tracking
+     * clipboard changes
+     */
     signals[TRACK]   = NEW_SIGNAL                 ("track"  , BOOLEAN);
 }
 
@@ -917,16 +1042,24 @@ g_paste_settings_init (GPasteSettings *self)
     GSettings *settings = priv->settings = g_settings_new (G_PASTE_SETTINGS_NAME);
 
     priv->history_name = NULL;
+    priv->launch_ui = NULL;
     priv->make_password = NULL;
     priv->pop = NULL;
     priv->show_history = NULL;
     priv->sync_clipboard_to_primary = NULL;
     priv->sync_primary_to_clipboard = NULL;
+    priv->upload = NULL;
+
+    priv->changed_signal = g_signal_connect (settings,
+                                             "changed",
+                                             G_CALLBACK (g_paste_settings_settings_changed),
+                                             self);
 
     g_paste_settings_private_set_element_size_from_dconf (priv);
     g_paste_settings_private_set_growing_lines_from_dconf (priv);
     g_paste_settings_private_set_history_name_from_dconf (priv);
     g_paste_settings_private_set_images_support_from_dconf (priv);
+    g_paste_settings_private_set_launch_ui_from_dconf (priv);
     g_paste_settings_private_set_make_password_from_dconf (priv);
     g_paste_settings_private_set_max_displayed_history_size_from_dconf (priv);
     g_paste_settings_private_set_max_history_size_from_dconf (priv);
@@ -943,32 +1076,29 @@ g_paste_settings_init (GPasteSettings *self)
     g_paste_settings_private_set_track_changes_from_dconf (priv);
     g_paste_settings_private_set_track_extension_state_from_dconf (priv);
     g_paste_settings_private_set_trim_items_from_dconf (priv);
-
-    priv->changed_signal = g_signal_connect (settings,
-                                             "changed",
-                                             G_CALLBACK (g_paste_settings_settings_changed),
-                                             self);
+    g_paste_settings_private_set_upload_from_dconf (priv);
 
     priv->shell_settings = NULL;
     priv->extension_enabled = FALSE;
 
-#if G_PASTE_CONFIG_ENABLE_EXTENSION
+#ifdef ENABLE_EXTENSION
     GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
     if (!source)
         return;
 
-    G_PASTE_CLEANUP_GSCHEMA_UNREF GSettingsSchema *schema = g_settings_schema_source_lookup (source, G_PASTE_SHELL_SETTINGS_NAME, TRUE);
+    g_autoptr (GSettingsSchema) schema = g_settings_schema_source_lookup (source, G_PASTE_SHELL_SETTINGS_NAME, TRUE);
+
     if (!schema)
         return;
 
     priv->shell_settings = g_settings_new (G_PASTE_SHELL_SETTINGS_NAME);
 
-    g_paste_settings_private_set_extension_enabled_from_dconf (priv);
-
     priv->shell_changed_signal = g_signal_connect (priv->shell_settings,
                                                   "changed::" G_PASTE_SHELL_ENABLED_EXTENSIONS_SETTING,
                                                   G_CALLBACK (g_paste_settings_shell_settings_changed),
                                                   self);
+
+    g_paste_settings_private_set_extension_enabled_from_dconf (priv);
 #endif
 }
 
